@@ -486,6 +486,7 @@ $VERSION = '1.32';
             '3 0 0' => 'Vivid Landscape',
             # have also seen '1 1 1', '2 2 4', '4 3 3', '4 4 4'
             # '0 0 14' and '0 0 42' - premium auto night shot (EX-Z2300)
+            # and '0 0 2' for Art HDR
         },
     },
     0x2089 => [ #PH
@@ -664,7 +665,7 @@ $VERSION = '1.32';
             47 => 'Painting', # (EX-2300)
             49 => 'Crayon Drawing', # (EX-2300)
             51 => 'Panorama', # (EX-ZR10)
-            52 => 'Art HDR', # (EX-ZR10)
+            52 => 'Art HDR', # (EX-ZR10,EX-Z3000)
         },
     },
     0x301c => { #3
