@@ -16,7 +16,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.11';
+$VERSION = '1.17';
 
 sub ProcessRicohText($$$);
 sub ProcessRicohRMETA($$$);
@@ -31,6 +31,8 @@ my %ricohLensIDs = (
     # (the exact lens model names used by Ricoh, except for a change in case)
     'RL1' => 'GR Lens A12 50mm F2.5 Macro',
     'RL2' => 'Ricoh Lens S10 24-70mm F2.5-4.4 VC',
+    'RL3' => 'Ricoh Lens P10 28-300mm F3.5-5.6 VC',
+    'RL5' => 'GR Lens A12 28mm F2.5',
 );
 
 %Image::ExifTool::Ricoh::Main = (
@@ -71,15 +73,11 @@ my %ricohLensIDs = (
         Name => 'PrintIM',
         Writable => 0,
         Description => 'Print Image Matching',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::PrintIM::Main',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::PrintIM::Main' },
     },
     0x1001 => {
         Name => 'ImageInfo',
-        SubDirectory => {
-            TagTable => 'Image::ExifTool::Ricoh::ImageInfo',
-        },
+        SubDirectory => { TagTable => 'Image::ExifTool::Ricoh::ImageInfo' },
     },
     0x1003 => {
         Name => 'Sharpness',
@@ -218,6 +216,7 @@ my %ricohLensIDs = (
             6 => 400,
             7 => 800,
             8 => 1600,
+            9 => 'Auto', #PH (? CX3)
         },
     },
     40 => {
@@ -234,6 +233,7 @@ my %ricohLensIDs = (
     },
 );
 
+# Ricoh subdirectory tags (ref PH)
 # NOTE: this subdir is not currently writable because the offsets would require
 # special code to handle the funny start location and base offset
 %Image::ExifTool::Ricoh::Subdir = (
@@ -243,26 +243,128 @@ my %ricohLensIDs = (
     # the significance of the following 2 dates is not known.  They are usually
     # within a month of each other, but I have seen differences of nearly a year.
     # Sometimes the first is more recent, and sometimes the second.
-    0x0004 => { #PH (NC)
+    0x0004 => { # (NC)
         Name => 'ManufactureDate1',
         Groups => { 2 => 'Time' },
         Writable => 'string',
         Count => 20,
     },
-    0x0005 => { #PH (NC)
+    0x0005 => { # (NC)
         Name => 'ManufactureDate2',
         Groups => { 2 => 'Time' },
         Writable => 'string',
         Count => 20,
     },
     # 0x000c - int32u[2] 1st number is a counter (file number? shutter count?) - PH
-    0x002c => { #PH (GXR)
+    # 0x0014 - int8u[338] - could contain some data related to face detection? - PH
+    # 0x0015 - int8u[2]: related to noise reduction?
+    0x001a => { #PH
+        Name => 'FaceInfo',
+        SubDirectory => { TagTable => 'Image::ExifTool::Ricoh::FaceInfo' },
+    },
+    0x0029 => {
+        Name => 'FirmwareInfo',
+        SubDirectory => { TagTable => 'Image::ExifTool::Ricoh::FirmwareInfo' },
+    },
+    0x002a => {
+        Name => 'NoiseReduction',
+        # this is the applied value if NR is set to "Auto"
+        Writable => 'int32u',
+        PrintConv => {
+            0 => 'Off',
+            1 => 'Weak',
+            2 => 'Strong',
+            3 => 'Max',
+        },
+    },
+    0x002c => { # (GXR)
         Name => 'SerialInfo',
         SubDirectory => { TagTable => 'Image::ExifTool::Ricoh::SerialInfo' },
     }
     # 0x000E ProductionNumber? (ref 2) [no. zero for most models - PH]
 );
 
+# face detection information (ref PH, CX4)
+%Image::ExifTool::Ricoh::FaceInfo = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    FIRST_ENTRY => 0,
+    DATAMEMBER => [ 181 ],
+    0xb5 => { # (should be int16u at 0xb4?)
+        Name => 'FacesDetected',
+        DataMember => 'FacesDetected',
+        RawConv => '$$self{FacesDetected} = $val',
+    },
+    0xb6 => {
+        Name => 'FaceDetectFrameSize',
+        Format => 'int16u[2]',
+    },
+    0xbc => {
+        Name => 'Face1Position',
+        Condition => '$$self{FacesDetected} >= 1',
+        Format => 'int16u[4]',
+        Notes => q{
+            left, top, width and height of detected face in coordinates of
+            FaceDetectFrameSize with increasing Y downwards
+        },
+    },
+    0xc8 => {
+        Name => 'Face2Position',
+        Condition => '$$self{FacesDetected} >= 2',
+        Format => 'int16u[4]',
+    },
+    0xd4 => {
+        Name => 'Face3Position',
+        Condition => '$$self{FacesDetected} >= 3',
+        Format => 'int16u[4]',
+    },
+    0xe0 => {
+        Name => 'Face4Position',
+        Condition => '$$self{FacesDetected} >= 4',
+        Format => 'int16u[4]',
+    },
+    0xec => {
+        Name => 'Face5Position',
+        Condition => '$$self{FacesDetected} >= 5',
+        Format => 'int16u[4]',
+    },
+    0xf8 => {
+        Name => 'Face6Position',
+        Condition => '$$self{FacesDetected} >= 6',
+        Format => 'int16u[4]',
+    },
+    0x104 => {
+        Name => 'Face7Position',
+        Condition => '$$self{FacesDetected} >= 7',
+        Format => 'int16u[4]',
+    },
+    0x110 => {
+        Name => 'Face8Position',
+        Condition => '$$self{FacesDetected} >= 8',
+        Format => 'int16u[4]',
+    },
+);
+
+# firmware version information (ref PH)
+%Image::ExifTool::Ricoh::FirmwareInfo = (
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    PROCESS_PROC => \&Image::ExifTool::ProcessBinaryData,
+    WRITE_PROC => \&Image::ExifTool::WriteBinaryData,
+    CHECK_PROC => \&Image::ExifTool::CheckBinaryData,
+    WRITABLE => 1,
+    0x00 => {
+        Name => 'FirmwareRevision',
+        Format => 'string[12]',
+    },
+    0x0c => {
+        Name => 'FirmwareRevision2',
+        Format => 'string[12]',
+    },
+);
+
 # serial/version number information written by GXR (ref PH)
 %Image::ExifTool::Ricoh::SerialInfo = (
     GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
@@ -375,7 +477,8 @@ my %ricohLensIDs = (
     GROUPS => { 0 => 'MakerNotes', 2 => 'Video' },
     ucmt => {
         Name => 'Comment',
-        ValueConv => '$_=$val; s/^Unicode//; tr/\0//d; s/\s+$//; $_',
+        # Ricoh writes a "Unicode" header even when text is ASCII (spaces anyway)
+        ValueConv => '$_=$val; s/^(Unicode\0|ASCII\0\0\0)//; tr/\0//d; s/\s+$//; $_',
     },
     mnrt => {
         Name => 'MakerNoteRicoh',
@@ -404,6 +507,7 @@ my %ricohLensIDs = (
     LensID => {
         SeparateTable => 'Ricoh LensID',
         Require => 'Ricoh:LensFirmware',
+        RawConv => '$val[0] ? $val[0] : undef',
         ValueConv => '$val=~s/\s*:.*//; $val',
         PrintConv => \%ricohLensIDs,
     },
@@ -435,7 +539,6 @@ sub ProcessRicohText($$$)
         $exifTool->Warn('Bad Ricoh maker notes');
         return 0;
     }
-    my $pos = 0;
     while ($data =~ m/([A-Z][a-z]{1,2})([0-9A-F]+);/sg) {
         my $tag = $1;
         my $val = $2;
@@ -573,7 +676,7 @@ interpret Ricoh maker notes EXIF meta information.
 
 =head1 AUTHOR
 
-Copyright 2003-2010, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2011, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
