@@ -12,15 +12,26 @@
 package Image::ExifTool::Samsung;
 
 use strict;
-use vars qw($VERSION);
+use vars qw($VERSION %samsungLensTypes);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.08';
+$VERSION = '1.09';
 
 sub WriteSTMN($$$);
 sub ProcessINFO($$$);
 
+%samsungLensTypes = (
+    0 => 'Built-in', #PH (EX1, WB2000)
+    1 => 'Samsung 30mm F2 Pancake',
+    2 => 'Samsung Zoom 18-55mm F3.5-5.6 OIS',
+    3 => 'Samsung Zoom 50-200mm F4-5.6 ED OIS',
+    # what about the non-OIS version of the 18-55,
+    # which was supposed to be available before the 20-50?
+    4 => 'Samsung 20-50mm F3.5-5.6 Compact Zoom', #PH
+    5 => 'Samsung 20mm F2.8 Pancake', #PH (guess)
+);
+
 # range of values for Formats used in encrypted information
 my %formatMinMax = (
     int16u => [ 0, 65535 ],
@@ -140,16 +151,7 @@ my %formatMinMax = (
         Name => 'LensType',
         Groups => { 2 => 'Camera' },
         Writable => 'int16u',
-        PrintConv => {
-            0 => 'Built-in', #PH (EX1, WB2000)
-            1 => 'Samsung 30mm F2 Pancake',
-            2 => 'Samsung Zoom 18-55mm F3.5-5.6 OIS',
-            3 => 'Samsung Zoom 50-200mm F4-5.6 ED OIS',
-            # what about the non-OIS version of the 18-55,
-            # which was supposed to be available before the 20-50?
-            4 => 'Samsung 20-50mm F3.5-5.6 Compact Zoom', #PH
-            5 => 'Samsung 20mm F2.8 Pancake', #PH (guess)
-        },
+        PrintConv => \%samsungLensTypes,
     },
     0xa004 => { #1
         Name => 'LensFirmware',
